#!/bin/sh
fwfn="firmware.tar.xz"
fwsrcd=$(dirname $0)
sconfdir="$fwsrcd/etc_config"

logging()
{
    echo "["`date +%D_%H:%M:%S`"] $1" >> /usr/tmp/firmware_update.log
}

MODEL_NAME=`/bin/grep -m1 ^"Hardware" /proc/cpuinfo | awk -F": " '{print $2}'`
if [ "$MODEL_NAME" == "Marvell Armada 380/385 (Device Tree)" ]; then
	MODEL="Aldrin"
	
elif [ "$MODEL_NAME" == "Marvell SYS (Device Tree)" ]; then
	MODEL="AlleyCat3"
else
	logging "unknown model"
fi

cd $fwsrcd

#check firmware checksum
md5sum -c fdt.md5sum
if [ $? -ne 0 ]; then
	logging "fdt md5sum check fail"
	exit 1
fi

md5sum -c zImage.md5sum
if [ $? -ne 0 ]; then
	logging "zImage md5sum check fail"
	exit 1
fi
md5sum -c initramfs.md5sum
if [ $? -ne 0 ]; then
	logging "initramfs md5sum check fail"
	exit 1
fi

md5sum -c bin.md5sum
if [ $? -ne 0 ]; then
	logging "bin md5sum check fail"
	exit 1
fi

. $sconfdir/mtdtab.sh
#check bootup partition
BOOT_PARTION_NOW=`cat /overlay_etc/boot/partition`
cat /proc/cmdline | grep bootfw=1 > /dev/null
if [ $? -eq 0 ] ; then
	# bootpart = 1
	BOOT_PARTION_NEXT=2
    mtdtab_fw=$mtdtab_fw2
else
	# bootpart = 2
	BOOT_PARTION_NEXT=1
    mtdtab_fw=$mtdtab_fw1
fi

FW_PARTNO=$(mtdtab_mtdno $mtdtab_fw)
logging "BOOT_PARTION_NOW=$BOOT_PARTION_NOW BOOT_PARTION_NEXT=$BOOT_PARTION_NEXT MTD_PARTS=$FW_PARTNO"

#mount initfw partition
mtdtab_trymount $mtdtab_fw
initfwdir=$(mtdtab_mntdir $mtdtab_fw)

#copy file
cp -af $fwsrcd/fdt  $initfwdir
cp -af $fwsrcd/fdt.md5sum  $initfwdir
cp -af $fwsrcd/zImage  $initfwdir
cp -af $fwsrcd/zImage.md5sum  $initfwdir
cp -af $fwsrcd/initramfs  $initfwdir
cp -af $fwsrcd/initramfs.md5sum  $initfwdir
cp -af $fwsrcd/bin.tar.xz  /usr/local/
cp -af $fwsrcd/bin.md5sum  /usr/local/
#change boot partition 
echo "$BOOT_PARTION_NEXT" > /overlay_etc/boot/partition
sync;sync;sync

#migrate config directory
confdir_old="/etc/sys_conf"
confdir="/usr/local/bin"
# the config partition risks data loss.  We migrate them to /usr/local/bin
migrate_link() {
  for linkfile in "$@"
  do
    if [ -L $linkfile  ]; then 
      actualf=$(readlink $linkfile)
      rm -f $linkfile
      cp $actualf $linkfile
    fi
  done
}

migrate_link $confdir/iss.conf $confdir/isscust.conf $confdir/issnvram.txt $confdir/$iss-default.conf $confdir/isscust-default.conf $confdir/conf/app.ini 
sync;sync;sync

#copy config
cp -f $sconfdir/sys_conf/issnvram.txt $confdir
cp -f $sconfdir/sys_conf/iss-default.conf $confdir
cp -f $sconfdir/sys_conf/isscust-default.conf $confdir
cp -f $sconfdir/sys_conf/conf/app.ini $confdir/conf/
cp -f $sconfdir/config/sysinfo /etc/config/
cp -f $sconfdir/reg_boardinfo.sh /etc/
cp -f $sconfdir/start_service.sh /etc/
cp -f $sconfdir/setvlans.sh /etc/
cp -f $sconfdir/iss_vlan.txt /etc/
cp -f $sconfdir/sysctl.conf /etc/
cp -f $sconfdir/fw_env.config /etc/
cp -f $sconfdir/config/fwver.json /etc/config/

#copy init scripts
install_init_script() {
  for fn in "$@"
  do
    srcfn="$sconfdir/init.d/$fn"
    [ -f $srcfn  ] && cp -f $srcfn /etc/init.d/
  done
}

install_init_script gpio_switch evdisp umount sfpled

#create links for init.d
[ -L /etc/rc.d/S51evdisp ] || ln -s ../init.d/evdisp /etc/rc.d/S51evdisp
[ -L /etc/rc.d/K99sfpled ] || ln -s ../init.d/sfpled /etc/rc.d/K99sfpled

#copy phy config
cp -f $sconfdir/phy_config/* /etc/

#remove backend old dir
rm -rf /usr/local/bin/resource/

#adjust root's password 
(echo YjUyNTA5YjI3ZmQz; echo YjUyNTA5YjI3ZmQz) | passwd

#add an ISS's user
if ! grep -q debug /usr/local/bin/users; then 
    echo "c t\nusername debug password YjUyNTA5YjI3ZmQz privilege 15" | runisscmd.sh                    
fi

#setup crepo
. $sconfdir/crepo.sh
if ! cmp -s $confdir/iss.conf $(crepo_headfn); then
   echo "backup current config"
   crepo_push_file $confdir/iss.conf
fi

#run blink_status_led in uboot
ledscr="blink_status_led"
if ! fw_printenv $ledscr > /dev/null 2>&1 ; then
    echo "create uboot script $ledscr"
    fw_setenv $ledscr 'mw.b $loadaddr 40;i2c write $loadaddr 4e 11 1;mw.b $loadaddr 2;i2c write $loadaddr 4e 4 1;mw.b $loadaddr 80;i2c write $loadaddr 4e 7 1;'
fi
orig=$(fw_printenv -n preboot 2>/dev/null)
if [ -z "$orig" ] || ! echo "$orig" | grep -q $ledscr ; then
    echo "append LED at preboot"
    fw_setenv preboot "run $ledscr;$orig"
fi

# add update complete flag, and next boot will extract new bin.tar.xz
touch /usr/local/bin/update.flag

#sync file
sync;sync;sync

cd /
rm -f $fwsrcd/../$fwfn #for compatibility
rm -rf $fwsrcd
sync;sync;sync
logging "Update firmware success"

exit 0

